"""
ice_pick.filters.regions
~~~~~~~~~~~~~~~~~~~~~~~~

This module contains a set of constants that can be used as region
filters. All the constants are based on AWS definitions.

Copyright 2014 Demand Media.

    Licensed under the Apache License, Version 2.0 (the "License");
    you may not use this file except in compliance with the License.
    You may obtain a copy of the License at

    http://www.apache.org/licenses/LICENSE-2.0

    Unless required by applicable law or agreed to in writing, software
    distributed under the License is distributed on an "AS IS" BASIS,
    WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
    See the License for the specific language governing permissions and
    limitations under the License.
"""


AP_NORTHEAST_1 = 'ap-northeast-1'  # Asia Pacific (Tokyo) Region
AP_SOUTHEAST_1 = 'ap-southeast-1'  # Asia Pacific (Singapore) Region
AP_SOUTHEAST_2 = 'ap-southeast-2'  # Asia Pacific (Sydney) Region
EU_WEST_1 = 'eu-west-1'  # EU (Ireland) Region
SA_EAST_1 = 'sa-east-1'  # South America (Sao Paulo) Region
US_EAST_1 = 'us-east-1'  # US East (Northern Virginia) Region
US_WEST_1 = 'us-west-1'  # US West (Northern California) Region
US_WEST_2 = 'us-west-2'  # US West (Oregon) Region
